# automatically generated by the FlatBuffers compiler, do not modify

# namespace: FBOutput

class PassMask(object):
    _img = 1
    _id = 2
    _category = 4
    _mask = 8
    _depth = 16
    _normals = 32
    _flow = 64
    _depth_simple = 128
    _albedo = 512

