# automatically generated by the FlatBuffers compiler, do not modify

# namespace: FBOutput

class JointType(object):
    none = 1
    fixed_joint = 2
    revolute = 4
    prismatic = 8
    spherical = 16

